#include <kernel/tty.h>
#include <kernel/idt.h>
#include <kernel/elf.h>
#include <kernel/pic.h>
#include <kernel/heap.h>
#include <kernel/task.h>
#include <kernel/paging.h>

#include <debugging.h>

/* Check if the compiler thinks you are targeting the wrong operating system. */
#if defined(__linux__)
#error "You are not using a cross-compiler, you will most certainly run into trouble"
#endif

uint8_t program[] = {
    0xcd, 0x80, // int 0x80
};

uint8_t data[] = "Hello From Userspace\n";


void kernel_main(void) 
{
    /* Initialize terminal interface */
    terminal_initialize();
    terminal_writestring("Hello from Kernelspace\n");

    idt_init();
    init_pic();
    heap_init();

    setup_tasks();
    setup_paging();

    asm("cli");

    run_program(program, sizeof(program), data, sizeof(data));

    asm("sti");

    while (true) {
        terminal_writestring("Hello from Kernelspace 1\n");
        terminal_writestring("Hello from Kernelspace 2\n");
    }
}
